package com.chelvc.cloud.vehicle.server.dao;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chelvc.cloud.vehicle.client.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantRankCountDTO;
import com.chelvc.cloud.vehicle.client.dto.MerchantRankDTO;
import com.chelvc.cloud.vehicle.client.param.LocationQueryPagingParam;
import com.chelvc.cloud.vehicle.client.param.LocationQueryParam;
import com.chelvc.cloud.vehicle.client.param.MerchantQueryParam;
import com.chelvc.cloud.vehicle.server.entity.Merchant;
import com.chelvc.framework.common.model.Pagination;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * 商家数据操作接口
 *
 * @author Woody
 * @date 2023/5/2
 */
@Mapper
public interface MerchantMapper extends BaseMapper<Merchant> {
    /**
     * 获取附近商家
     *
     * @param param 查询参数
     * @param size  商家数量
     * @return 商家信息列表
     */
    List<MerchantDTO> listNearbyMerchants(@Param("param") LocationQueryParam param, @Param("size") int size);


    /**
     * 分页获取附近商家
     *
     * @param param 查询参数
     * @return 商家信息分页列表
     */
    Page<MerchantDTO> listNearbyMerchantPaging(IPage<LocationQueryPagingParam> page,@Param("param")LocationQueryPagingParam param);

    /**
     * 获取推荐商家
     *
     * @param param 查询参数
     * @return 商家信息分页列表
     */
    Page<MerchantDTO> listRecommendMerchantPaging(IPage<LocationQueryPagingParam> page,@Param("param")LocationQueryPagingParam param);


    /**
     * 分页获取推荐商家
     *
     * @param param 查询参数
     * @param size  商家数量
     * @return 商家信息列表
     */
    List<MerchantDTO> listRecommendMerchants(@Param("param") LocationQueryParam param, @Param("size") int size);

    /**
     * 查询商家
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    List<MerchantDTO> listSimpleMerchants(MerchantQueryParam param);

    /**
     * 获取商家详情
     *
     * @param id 商家ID
     * @param param 查询参数
     * @return 商家详情
     */
    Merchant getMerchantById(@Param("id") Long id, @Param("param") LocationQueryParam param);

    /**
     * 商家是否被收藏
     * @param merchantId
     * @return
     */
    Integer merchantFavorite(@Param("merchantId") Long merchantId,@Param("userId") Long userId);

    /**
     * 商家排名
     * @param monthStartLocalTime
     * @param categoryId
     * @param longitude
     * @param latitude
     * @param regionId
     * @param level
     * @return
     */
    List<MerchantRankDTO> listRankMerchants(@Param("merchantId")LocalDateTime monthStartLocalTime,
                                            @Param("categoryId")Long categoryId,
                                            @Param("regionId")Integer regionId,
                                            @Param("longitude")Double longitude,
                                            @Param("latitude")Double latitude, @Param("level")int level);

    /**
     * 查询用户名下得店铺
     * @param userId 用户id
     * @return
     */
    List<Long> getMerchantIds(Long userId);

    /**
     * 更新商家营业状态
     * @param merchantId 商家ID
     * @param businessStatus 营业状态
     */
    public void updateMerchantBusinessStatus(@Param("merchantId")Long merchantId,
                                             @Param("businessStatus")String businessStatus,
                                             @Param("automatic")String automatic);

    List<MerchantRankCountDTO> rankMerchants(@Param("monthStartLocalTime") LocalDateTime monthStartLocalTime,
                                             @Param("monthEndLocalTime") LocalDateTime monthEndLocalTime,
                                             @Param("regionId") Integer regionId,
                                             @Param("limit") Integer limit,
                                             @Param("level") Integer level,
                                             @Param("categoryId") Long categoryId);

    BigDecimal queryTotalCommissionAmountByLastMonth(@Param("lastMonthStartLocalTime") LocalDateTime lastMonthStartLocalTime,
                                                     @Param("lastMonthEndLocalTime") LocalDateTime lastMonthEndLocalTime,
                                                     @Param("regionId") Integer regionId,
                                                     @Param("level") Integer level,
                                                     @Param("categoryId") Long categoryId);

    /**
     * 获取商家营业状况
     * @return
     */
    Map<String,Object> operatConditions(@Param("merchantId")Long merchantId);

    /**
     * 经营报表
     * @return 经营报表信息
     */
    List<Map<String,Object>> operatReportByYear(@Param("merchantId")Long merchantId,@Param("year")Integer year);

    /**
     * 经营报表
     * @return 经营报表信息
     */
    List<Map<String,Object>> operatReportByMonth(@Param("merchantId")Long merchantId,@Param("year")Integer year,@Param("month")Integer month);


    /**
     * 获取商家入驻统计
     * @param year 查询年期
     * @return
     */
    List<Map<String,Object>> getAddMerchantReportByYear(@Param("year") Integer year);


    /**
     * 获取商家入驻统计
     * @param year 查询年期
     * @return
     */
    List<Map<String,Object>> getAddMerchantReportByMonth(@Param("year") Integer year,@Param("month")Integer month);



    /**
     * 获取用户注册统计
     * @param year 查询年期
     * @param month 查询月份
     * @return
     */
    Map<String,Object> getAddMerchantReport(@Param("year") Integer year,
                                             @Param("month") Integer month,@Param("day") Integer day);
}
